﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using Telerik.Windows.Controls.Data.DataFilter;
using System.Collections;
using System.Reflection;
using Telerik.Windows.Controls;
using MultOptionDataFilter.Contracts;
using Telerik.Windows.Controls.Primitives;

namespace MultOptionDataFilter.Controls
{
    public class MultiOptionFilter : Control
    {
        #region Private fields

        private const string NO_ITEMS_SELECTED_TEXT = "No items selected";
        private SimpleFilterViewModel _simpleFilterViewModel;
        protected readonly Dictionary<object, int> _currentOptions;
        protected readonly Dictionary<object, object> _currentItemsValue;
        private Button OpenPopupButton;
        private Popup Popup;

        #endregion

        #region Dependency properties

        public ICommand FilterChangedCommand
        {
            get { return (ICommand)GetValue(FilterChangedCommandProperty); }
            set { SetValue(FilterChangedCommandProperty, value); }
        }

        // Using a DependencyProperty as the backing store for FilterChangedCommand.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty FilterChangedCommandProperty =
            DependencyProperty.Register("FilterChangedCommand", typeof(ICommand), typeof(MultiOptionFilter), new PropertyMetadata(null));



        public object FilterChangedCommandParameter
        {
            get { return (object)GetValue(FilterChangedCommandParameterProperty); }
            set { SetValue(FilterChangedCommandParameterProperty, value); }
        }

        // Using a DependencyProperty as the backing store for FilterChangedCommandParameter.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty FilterChangedCommandParameterProperty =
            DependencyProperty.Register("FilterChangedCommandParameter", typeof(object), typeof(MultiOptionFilter), new PropertyMetadata(null));



        public IEnumerable Source
        {
            get { return (IEnumerable)GetValue(SourceProperty); }
            set { SetValue(SourceProperty, value); }
        }

        // Using a DependencyProperty as the backing store for Source.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty SourceProperty =
            DependencyProperty.Register("Source", typeof(IEnumerable), typeof(MultiOptionFilter), new PropertyMetadata(null, OnSourceChanged));

        private static void OnSourceChanged(DependencyObject owner, DependencyPropertyChangedEventArgs e)
        {
            MultiOptionFilter option = owner as MultiOptionFilter;
            if (option != null)
                option.LoadOptions();
        }

        public string FilterStatus
        {
            get { return (string)GetValue(FilterStatusProperty); }
            set { SetValue(FilterStatusProperty, value); }
        }

        // Using a DependencyProperty as the backing store for FilterStatus.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty FilterStatusProperty =
            DependencyProperty.Register("FilterStatus", typeof(string), typeof(MultiOptionFilter), new PropertyMetadata(NO_ITEMS_SELECTED_TEXT));



        public ObservableCollection<FilterOptionSelecteable> OptionsView
        {
            get { return (ObservableCollection<FilterOptionSelecteable>)GetValue(OptionsViewProperty); }
            set { SetValue(OptionsViewProperty, value); }
        }

        // Using a DependencyProperty as the backing store for OptionsView.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty OptionsViewProperty =
            DependencyProperty.Register("OptionsView", typeof(ObservableCollection<FilterOptionSelecteable>), typeof(MultiOptionFilter), new PropertyMetadata(null));

        #endregion


        public MultiOptionFilter()
        {
            this.DefaultStyleKey = typeof(MultiOptionFilter);
            OptionsView = new ObservableCollection<FilterOptionSelecteable>();
            _currentItemsValue = new Dictionary<object, object>();
            _currentOptions = new Dictionary<object, int>();
        }

        #region Control overrides

        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();
            IMultiOptionFilterSource viewModel = this.FindAncestor<RadDataFilter>().DataContext as IMultiOptionFilterSource;
            FilterChangedCommand = viewModel.MultiOptionFilterChangedCommand;
            Source = viewModel.Source;

            Popup = GetTemplateChild("PART_Popup") as Popup;

            OpenPopupButton = GetTemplateChild("PART_OpenPopup") as Button;

            if (OpenPopupButton != null)
                OpenPopupButton.Click += (s, e) => Popup.IsOpen = !Popup.IsOpen;
            Loaded += OnLoaded;
        }

        #endregion

        #region Private methods

        protected SimpleFilterViewModel SimpleFilterViewModel
        {
            get
            {
                if (_simpleFilterViewModel == null)
                    _simpleFilterViewModel = DataContext as SimpleFilterViewModel;
                return _simpleFilterViewModel;
            }
        }
        private void OnLoaded(object sender, RoutedEventArgs e)
        {
            object selection;
            List<FilterOptionSelecteable> selectedValues = null;
            if (SimpleFilterViewModel.Descriptor.TryGetDirtyValue(out selection))
                selectedValues = selection as List<FilterOptionSelecteable>;
            LoadOptions();
            foreach (var option in OptionsView)
                option.IsCheckedChanged -= OptionCheckedChanged;
            if (selectedValues != null)
            {
                foreach (FilterOptionSelecteable option in selectedValues)
                {
                    if (option.IsChecked)
                    {
                        var opt = (from o in OptionsView where o.Value.Equals(option.Value) select o).SingleOrDefault();
                        if (opt != null)
                            opt.IsChecked = true;
                    }

                }

            }
            foreach (var option in OptionsView)
                option.IsCheckedChanged += OptionCheckedChanged;
        }

        private void LoadOptions()
        {
            if (SimpleFilterViewModel == null)
                return;
            OptionsView.Clear();
            _currentItemsValue.Clear();
            _currentOptions.Clear();
            if (SimpleFilterViewModel.MemberType.IsEnum)
            {
                var ff = SimpleFilterViewModel.MemberType.GetFields(BindingFlags.Public | BindingFlags.Static);
                foreach (var fi in ff)
                {
                    var value = Enum.Parse(SimpleFilterViewModel.MemberType, fi.Name, false);
                    _currentOptions.Add(value, 1);
                    OptionsView.Add(new FilterOptionSelecteable(value, false, fi.Name));
                }
            }
            else if (Source != null)
            {
                foreach (var item in Source)
                {
                    object propertyOwner;
                    var value = GetPropertyValue(item, out propertyOwner);
                    if (value != null && !_currentOptions.ContainsKey(value))
                    {
                        _currentOptions.Add(value, 1);
                        OptionsView.Add(new FilterOptionSelecteable(value, false, value));
                    }
                    else if (value != null)
                        _currentOptions[value] = (_currentOptions[value] + 1);
                }
            }

        }

        private void OptionCheckedChanged(object sender, FilterOptionCheckedChangedEventArgs e)
        {
            if (SimpleFilterViewModel != null)
            {
                List<FilterOptionSelecteable> selectedOption = new List<FilterOptionSelecteable>();
                foreach (var option in OptionsView)
                {
                    if (option.IsChecked)
                        selectedOption.Add(new FilterOptionSelecteable(option.Value, option.IsChecked, option.DisplayValue));
                }
                SimpleFilterViewModel.Descriptor.SetDirtyValue(selectedOption);
                if (this.FilterChangedCommand != null)
                {
                    if (FilterChangedCommand.CanExecute(FilterChangedCommandParameter))
                        FilterChangedCommand.Execute(FilterChangedCommandParameter);
                    if (selectedOption.Count == 0)
                        FilterStatus = NO_ITEMS_SELECTED_TEXT;
                    else
                        FilterStatus = string.Format("{0} items selected", selectedOption.Count);
                }
            }
        }

        protected virtual object GetPropertyValue(object candidate, out object propertyOwner)
        {
            propertyOwner = candidate;
            try
            {
                //Si no es defineix PropertyPath, la propietat és l'objecte candidat per si mateix
                if (string.IsNullOrEmpty(SimpleFilterViewModel.Member))
                    return candidate;
                //S'obté la ruta del graph de l'objecte per accedir a la propietat
                var accessors = SimpleFilterViewModel.Member.Split(new[] { "." }, StringSplitOptions.RemoveEmptyEntries);
                var type = candidate.GetType();
                var obj = candidate;
                PropertyInfo pi;
                //Es navega pel graph de l'objecte fins arribar a la instància que conté directament
                //la propietat i, en última instància, la propietat.
                for (int i = 0; i < accessors.Length; i++)
                {
                    pi = type.GetProperty(accessors[i]);
                    propertyOwner = obj;
                    obj = pi.GetValue(obj, null);
                    type = pi.PropertyType;
                }
                return obj;
            }
            catch
            {
            }
            return null;
        }

        #endregion
    }
}
