﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Telerik.Windows.Controls;
using MultOptionDataFilter.Contracts;
using System.Collections.Generic;
using Telerik.Windows.Controls.Data.DataFilter;
using System.ComponentModel.DataAnnotations;
using Telerik.Windows.Data;
using System.ComponentModel;
using MultOptionDataFilter.Controls;
using System.Windows.Data;

namespace MultOptionDataFilter.ViewModel
{
    public class ViewModel : ViewModelBase, IMultiOptionFilterSource
    {

        public ViewModel()
        {
            Items = new List<ViewModelItem>();
            for (int i = 0; i < 50; i++ )
            {
                Items.Add(new ViewModelItem(i, string.Format("Value {0}", i), (ItemType)(i % 3)));
            }

            FilterProperties = new ItemPropertyDefinitionCollection();

            var type = typeof(FilterMetadata);
            var pis = type.GetProperties();

            foreach (var pi in pis)
            {
                var displayAtt = pi.GetCustomAttributes(typeof(DisplayAttribute), false)[0] as DisplayAttribute;
                FilterProperties.Add(new ItemPropertyDefinition(displayAtt.Name, pi.PropertyType, displayAtt.ShortName));
            }
            ItemsView = new CollectionViewSource { Source = Items };
            MultiOptionFilterChangedCommand = new DelegateCommand(o => UpdateFilter());
        }

        #region IMultiOptionFilterSource Members

        public System.Collections.IEnumerable Source
        {
            get { return Items; }
        }

        public ICommand MultiOptionFilterChangedCommand { get; private set; }

        #endregion

        private List<ViewModelItem> _items;

        public List<ViewModelItem> Items
        {
            get
            {
                return this._items;
            }
            set
            {
                if (this._items != value)
                {
                    this._items = value;
                    this.OnPropertyChanged("Items");
                }
            }
        }

        public CollectionViewSource ItemsView { get; private set; }

        public ItemPropertyDefinitionCollection FilterProperties { get; private set; }

        private CompositeFilterDescriptorCollection _advancedFilters;

        private Predicate<object> _advancedFilter;
        public Predicate<object> AdvancedFilter
        {
            get { return _advancedFilter; }
        }

        public CompositeFilterDescriptorCollection AdvancedFilters
        {
            get { return _advancedFilters; }
            set
            {
                if (_advancedFilters != value)
                {
                    if (_advancedFilter != null)
                        UnSubscribeToAdvancedFilter(_advancedFilters);
                    _advancedFilters = value;
                    SubscribeToAdvancedFilter(_advancedFilters);
                    OnPropertyChanged("AdvancedFilter");
                }
            }
        }

        private void UnSubscribeToAdvancedFilter(CompositeFilterDescriptorCollection compositeFilter)
        {
            compositeFilter.PropertyChanged += FilterPropertyChanged;
            compositeFilter.ItemChanged += CompositeFilterItemChanged;
            compositeFilter.CollectionChanged += CompositeFilterCollectionChanged;
        }

        private void SubscribeToAdvancedFilter(CompositeFilterDescriptorCollection compositeFilter)
        {
            compositeFilter.PropertyChanged += FilterPropertyChanged;
            compositeFilter.ItemChanged += CompositeFilterItemChanged;
            compositeFilter.CollectionChanged += CompositeFilterCollectionChanged;
        }

        private void FilterPropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == "LogicalOperator")
                UpdateFilter();
        }

        void CompositeFilterItemChanged(object sender, ItemChangedEventArgs<IFilterDescriptor> e)
        {
            if (e.PropertyName == "Member")
            {
                foreach (var filter in sender as FilterDescriptorCollection)
                {
                    ReleaseFilterDirtyValue(filter);
                }
                UpdateFilter();
            }
            if (e.PropertyName == "Value" || e.PropertyName == "Operator")
                UpdateFilter();
        }

        private void ReleaseFilterDirtyValue(IFilterDescriptor filter)
        {
            if (filter is FilterDescriptor)
                (filter as FilterDescriptor).Release();
            else
            {
                foreach (var childFilter in (filter as CompositeFilterDescriptor).FilterDescriptors)
                    ReleaseFilterDirtyValue(childFilter);
            }
        }

        void CompositeFilterCollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            switch (e.Action)
            {
                case System.Collections.Specialized.NotifyCollectionChangedAction.Remove:
                    UpdateFilter();
                    if (e.OldItems != null)
                    {
                        foreach (var item in e.OldItems)
                        {
                            var filterDescriptor = item as FilterDescriptor;
                            if (filterDescriptor != null)
                                filterDescriptor.Release();
                        }
                    }
                    break;
                case System.Collections.Specialized.NotifyCollectionChangedAction.Reset:
                    UpdateFilter();
                    FilterDescriptorExtensions.ReleaseAll();
                    break;

            }
        }

        private void UpdateFilter()
        {
            var filter = GetAdvancedFilter();
            if (ItemsView.View.Filter != filter)
                ItemsView.View.Filter = filter;
            else
                ItemsView.View.Refresh();
        }

        private Predicate<object> GetAdvancedFilter()
        {
            Predicate<object> result = null;
            if (AdvancedFilters != null)
            {
                foreach (var f in AdvancedFilters)
                {
                    Predicate<object> subfilter;
                    if (f is CompositeFilterDescriptor)
                        subfilter = GetCompositeFilter(f as CompositeFilterDescriptor);
                    else
                        subfilter = GetFilter(f as FilterDescriptor);
                    result = AdvancedFilters.LogicalOperator == FilterCompositionLogicalOperator.And ? result.And(subfilter) : result.Or(subfilter);
                }
            }
            return result;
        }
        private static Predicate<object> GetCompositeFilter(CompositeFilterDescriptor compositeFilterDescriptor)
        {
            Predicate<object> filter = null;
            foreach (var f in compositeFilterDescriptor.FilterDescriptors)
            {
                Predicate<object> subfilter;
                if (f is CompositeFilterDescriptor)
                    subfilter = GetCompositeFilter(f as CompositeFilterDescriptor);
                else
                    subfilter = GetFilter(f as FilterDescriptor);
                filter = compositeFilterDescriptor.LogicalOperator == FilterCompositionLogicalOperator.And ? filter.And(subfilter) : filter.Or(subfilter);
            }
            return filter;
        }

        private static Predicate<object> GetFilter(FilterDescriptor filterDescriptor)
        {
            object tmp;
            if ((filterDescriptor.Operator == FilterOperator.IsEqualTo || filterDescriptor.Operator == FilterOperator.IsNotEqualTo) && filterDescriptor.TryGetDirtyValue(out tmp))
            {
                var selectedValues = tmp as List<FilterOptionSelecteable>;
                Predicate<object> compositeFilter = null;
                foreach (var option in selectedValues)
                {
                    var f = GetValueFilter(filterDescriptor, option.Value);
                    if (filterDescriptor.Operator == FilterOperator.IsEqualTo)
                        compositeFilter = compositeFilter.Or(f);
                    else
                        compositeFilter = compositeFilter.And(f);
                }
                return compositeFilter;
            }
            if (filterDescriptor.Value == FilterDescriptor.UnsetValue)
                return null;
            return GetValueFilter(filterDescriptor, filterDescriptor.Value);

        }

        private static Predicate<object> GetValueFilter(FilterDescriptor filterDescriptor, object value)
        {
            switch (filterDescriptor.Member)
            {
                case "Property1":
                    switch (filterDescriptor.Operator)
                    {
                        case FilterOperator.IsGreaterThan:
                            return new Predicate<object>(o => (o as ViewModelItem).Property1 > (int)value);
                        case FilterOperator.IsGreaterThanOrEqualTo:
                            return new Predicate<object>(o => (o as ViewModelItem).Property1 >= (int)value);
                        case FilterOperator.IsLessThan:
                            return new Predicate<object>(o => (o as ViewModelItem).Property1 < (int)value);
                        case FilterOperator.IsLessThanOrEqualTo:
                            return new Predicate<object>(o => (o as ViewModelItem).Property1 <= (int)value);
                        case FilterOperator.IsEqualTo:
                            return new Predicate<object>(o => (o as ViewModelItem).Property1 == (int)value);
                        case FilterOperator.IsNotEqualTo:
                            return new Predicate<object>(o => (o as ViewModelItem).Property1 != (int)value);
                    }
                    break;
                case "Property2":
                    switch (filterDescriptor.Operator)
                    {
                        case FilterOperator.Contains:
                            return new Predicate<object>(o => (o as ViewModelItem).Property2.Contains((string)value));
                        case FilterOperator.DoesNotContain:
                            return new Predicate<object>(o => (o as ViewModelItem).Property2.Contains((string)value) == false);
                        case FilterOperator.EndsWith:
                            return new Predicate<object>(o => (o as ViewModelItem).Property2.EndsWith((string)value) == true);
                        case FilterOperator.IsContainedIn:
                            return new Predicate<object>(o => ((string)value).Contains((o as ViewModelItem).Property2));
                        case FilterOperator.IsEqualTo:
                            return new Predicate<object>(o => (o as ViewModelItem).Property2 == (string)value);
                        case FilterOperator.IsNotContainedIn:
                            return new Predicate<object>(o => ((string)value).Contains((o as ViewModelItem).Property2) == false);
                        case FilterOperator.IsNotEqualTo:
                            return new Predicate<object>(o => (o as ViewModelItem).Property2 != (string)value);
                        case FilterOperator.StartsWith:
                            return new Predicate<object>(o => (o as ViewModelItem).Property2.StartsWith((string)value));
                    }
                    break;
                case "ItemType":
                    switch (filterDescriptor.Operator)
                    {
                        case FilterOperator.IsEqualTo:
                            return new Predicate<object>(o => (o as ViewModelItem).ItemType == (ItemType)value);
                        case FilterOperator.IsNotEqualTo:
                            return new Predicate<object>(o => (o as ViewModelItem).ItemType != (ItemType)value);
                    }
                    break;
            }
            return null;
        }
    }
}
